% P30D2nq.m SNR^2NQ
% COPYRIGHT:  Aprend Technology, 2011. All rights reserved.
 
%% init vars
post_title = 'P30D2nq';
dirname = ['G:\Projects\Blog\Posts\' post_title '\'];

%% Create sprectrum data for Monte Carlo simulations
tube_voltage = 80; % keV  
[specnum,xray_energies] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
specdat.egys = xray_energies;
specdat.specnum = specnum;
specdat.specnum_norm = specnum/trapz(specdat.egys,specnum); % normalize--will set total photons later
        % create the inverse cumulative spectrum for monte carlo
spinv =  SpectrumCumulativeInverse(specdat);
        % parameters for theoretical calculations              
Ebar = sum(xray_energies.*specdat.specnum_norm);
E2bar = sum((xray_energies.^2).*specdat.specnum_norm);
F = E2bar/Ebar^2;

%% Simulate NQ data with pileup
nMCtrials = 300;   %  Royston test does not work for large datasets
nruns = 10; % number of generate MC data/compute Hs runs
nlambdas = 20; % lambdas for Royston test
ndata_types = 2; % N and Q    
nqdat = zeros(nlambdas,nMCtrials,ndata_types);    
lambdas = geospace(1,1000,nlambdas);
HsNQlin = zeros(nruns,nlambdas); % hypothesis test results-- "1" if normal
HsNQlog = zeros(nruns,nlambdas); % hypothesis test results-- "1" if normal
alpha = 0.01;
for krun = 1:nruns
    for klambda = 1:nlambdas
        lambda = lambdas(klambda);
        ns = poissrnd(lambda, nMCtrials);
        for kt = 1:nMCtrials
            photon_energies = interp1(spinv.ys,spinv.specuminv,rand(1,ns(kt)));
            Q = sum(photon_energies);
            nqdat(klambda,kt,:) = [ns(kt) Q];
        end
    end
    
        % run Royston's test with these data
    for klambda = 1:nlambdas
            dat = squeeze(nqdat(klambda,:,:));
            stats_roy = roystest(dat,'alpha',alpha);
            HsNQlin(krun,klambda) = stats_roy.H;
            stats_roy = roystest(log(dat),'alpha',alpha);
            HsNQlog(krun,klambda) = stats_roy.H;
    end
   fprintf('.')
   if mod(krun,80)==0
       fprintf('\n');
   end
end % for kruns
for k=1:5,beep,pause(0.1),end
fprintf('\ndone\n');

%% plot Q mean, variance and NQ covariance vs lambda
qbars = zeros(nlambdas,1);  
qvars = zeros(nlambdas,1);
nqcov = zeros(nlambdas,1);
for klambda = 1:nlambdas
    qbars(klambda) = mean(squeeze(nqdat(klambda,:,2)));
    qvars(klambda) = var(squeeze(nqdat(klambda,:,2)));
    C = cov(squeeze(nqdat(klambda,:,:)));
    nqcov(klambda) = C(1,2);
end
fh = figure;
fontsize = 14;
plot(lambdas,qbars,'ok',lambdas,lambdas*Ebar,'-k','linewidth',1.5)
hold on,plot(lambdas,qvars,'*r',lambdas,lambdas*E2bar,'-r','linewidth',1.5),hold off
hold on,plot(lambdas,nqcov,'+b',lambdas,lambdas*Ebar,'-b','linewidth',1.5),hold off
set(gca,'Xscale','log','Yscale','log','fontsize',fontsize,'linewidth',1.5)
xlabel('mean counts','fontsize',fontsize)
ylabel('Q stats','fontsize',fontsize)
strs = {'mean-MC','mean-thry','var-MC','var-thry','cov-MC','cov-thry'};
legend(strs,'location','southeast')
set(fh,'color','w')

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Qstats';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',5,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot normal test results
fh = figure;
fontsize = 14;
plot(lambdas,mean(HsNQlin),'-ok','linewidth',1.5)
hold on,plot(lambdas,mean(HsNQlog),'-*r','linewidth',1.5),hold off
set(gca,'Xscale','log','fontsize',fontsize,'linewidth',1.5)
xlabel('mean counts','fontsize',fontsize)
ylabel('probability distribution is normal','fontsize',fontsize)
legend('linear','log','location','southeast')
set(fh,'color','w')

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'MultiVarNormalProbs';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',5,'LockAxes',0,'color','cmyk');
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi